@@ -42,17 +42,8 @@ extern "C" {
 enum connman_element_type {
 	CONNMAN_ELEMENT_TYPE_UNKNOWN    = 0,
 	CONNMAN_ELEMENT_TYPE_ROOT       = 1,
-	CONNMAN_ELEMENT_TYPE_PROFILE    = 2,
 	CONNMAN_ELEMENT_TYPE_DEVICE     = 3,
 	CONNMAN_ELEMENT_TYPE_NETWORK    = 4,
-	CONNMAN_ELEMENT_TYPE_SERVICE    = 5,
-	CONNMAN_ELEMENT_TYPE_IPV4       = 7,
-	CONNMAN_ELEMENT_TYPE_IPV6       = 8,
-	CONNMAN_ELEMENT_TYPE_DHCP       = 9,
-	CONNMAN_ELEMENT_TYPE_BOOTP      = 10,
-	CONNMAN_ELEMENT_TYPE_ZEROCONF   = 11,
-	CONNMAN_ELEMENT_TYPE_CONNECTION = 42,
-	CONNMAN_ELEMENT_TYPE_VENDOR     = 10000,
 };
 
 enum connman_element_state {
@@ -65,8 +56,6 @@ enum connman_element_state {
 enum connman_element_error {
 	CONNMAN_ELEMENT_ERROR_UNKNOWN        = 0,
 	CONNMAN_ELEMENT_ERROR_FAILED         = 1,
-	CONNMAN_ELEMENT_ERROR_DHCP_FAILED    = 2,
-	CONNMAN_ELEMENT_ERROR_CONNECT_FAILED = 3,
 };
 
 struct connman_driver;
@@ -97,29 +86,6 @@ struct connman_element {
 	};
 
 	GHashTable *properties;
-
-	gchar *hostname;
-	gchar *domainname;
-
-	struct {
-		enum connman_ipconfig_method method;
-		gchar *address;
-		gchar *netmask;
-		gchar *gateway;
-		gchar *network;
-		gchar *broadcast;
-		gchar *nameserver;
-		gchar *timeserver;
-		gchar *pac;
-	} ipv4;
-
-	struct {
-		enum connman_ipconfig_method method;
-		gchar *address;
-		int prefix_len;
-		gchar *gateway;
-		gchar *network;
-	} ipv6;
 };
 
 struct connman_element *connman_element_create(const char *name);
