@@ -51,6 +51,8 @@ enum connman_network_error {
 	CONNMAN_NETWORK_ERROR_UNKNOWN         = 0,
 	CONNMAN_NETWORK_ERROR_ASSOCIATE_FAIL  = 1,
 	CONNMAN_NETWORK_ERROR_CONFIGURE_FAIL  = 2,
+	CONNMAN_NETWORK_ERROR_INVALID_KEY     = 3,
+	CONNMAN_NETWORK_ERROR_CONNECT_FAIL    = 4,
 };
 
 #define CONNMAN_NETWORK_PRIORITY_LOW      -100
@@ -76,6 +78,7 @@ void connman_network_set_group(struct connman_network *network,
 						const char *group);
 const char *connman_network_get_group(struct connman_network *network);
 
+connman_bool_t connman_network_get_connecting(struct connman_network *network);
 int connman_network_set_available(struct connman_network *network,
 						connman_bool_t available);
 connman_bool_t connman_network_get_available(struct connman_network *network);
@@ -94,9 +97,14 @@ void connman_network_set_ipv4_method(struct connman_network *network,
 					enum connman_ipconfig_method method);
 void connman_network_set_ipv6_method(struct connman_network *network,
 					enum connman_ipconfig_method method);
-
-int connman_network_set_address(struct connman_network *network,
-				const void *address, unsigned int size);
+int connman_network_set_ipaddress(struct connman_network *network,
+				struct connman_ipaddress *ipaddress);
+int connman_network_set_nameservers(struct connman_network *network,
+				const char *nameservers);
+int connman_network_set_domain(struct connman_network *network,
+			             const char *domain);
+int connman_network_set_pac(struct connman_network *network,
+				const char *pac);
 int connman_network_set_name(struct connman_network *network,
 							const char *name);
 int connman_network_set_strength(struct connman_network *network,
