@@ -46,6 +46,7 @@ struct connman_device {
 	char *address;
 	char *interface;
 	char *ident;
+	char *path;
 	int phyindex;
 	unsigned int connections;
 	guint scan_timeout;
@@ -440,6 +441,7 @@ static void device_destruct(struct connman_element *element)
 	g_free(device->name);
 	g_free(device->address);
 	g_free(device->interface);
+	g_free(device->path);
 
 	g_free(device->last_network);
 
@@ -462,6 +464,7 @@ struct connman_device *connman_device_create(const char *node,
 	struct connman_device *device;
 	const char *str;
 	enum connman_service_type service_type;
+	connman_bool_t bg_scan;
 
 	DBG("node %s type %d", node, type);
 
@@ -471,6 +474,8 @@ struct connman_device *connman_device_create(const char *node,
 
 	DBG("device %p", device);
 
+	bg_scan = connman_setting_get_bool("BackgroundScanning");
+
 	__connman_element_initialize(&device->element);
 
 	device->element.name = g_strdup(node);
@@ -484,8 +489,6 @@ struct connman_device *connman_device_create(const char *node,
 		connman_element_set_string(&device->element,
 					CONNMAN_PROPERTY_ID_TYPE, str);
 
-	device->element.ipv4.method = CONNMAN_IPCONFIG_METHOD_DHCP;
-
 	device->type = type;
 	device->name = g_strdup(type2description(device->type));
 
@@ -499,27 +502,20 @@ struct connman_device *connman_device_create(const char *node,
 
 	switch (type) {
 	case CONNMAN_DEVICE_TYPE_UNKNOWN:
+	case CONNMAN_DEVICE_TYPE_ETHERNET:
+	case CONNMAN_DEVICE_TYPE_WIMAX:
+	case CONNMAN_DEVICE_TYPE_BLUETOOTH:
+	case CONNMAN_DEVICE_TYPE_CELLULAR:
+	case CONNMAN_DEVICE_TYPE_GPS:
+	case CONNMAN_DEVICE_TYPE_GADGET:
 	case CONNMAN_DEVICE_TYPE_VENDOR:
 		device->scan_interval = 0;
 		break;
-	case CONNMAN_DEVICE_TYPE_ETHERNET:
 	case CONNMAN_DEVICE_TYPE_WIFI:
-		device->scan_interval = 300;
-		break;
-	case CONNMAN_DEVICE_TYPE_WIMAX:
-		device->scan_interval = 0;
-		break;
-	case CONNMAN_DEVICE_TYPE_BLUETOOTH:
-		device->scan_interval = 0;
-		break;
-	case CONNMAN_DEVICE_TYPE_GPS:
-		device->scan_interval = 0;
-		break;
-	case CONNMAN_DEVICE_TYPE_CELLULAR:
-		device->scan_interval = 0;
-		break;
-	case CONNMAN_DEVICE_TYPE_GADGET:
-		device->scan_interval = 0;
+		if (bg_scan == TRUE)
+			device->scan_interval = 300;
+		else
+			device->scan_interval = 0;
 		break;
 	}
 
@@ -798,7 +794,7 @@ int __connman_device_disconnect(struct connman_device *device)
 			 * will fail.  There is nothing to be gained by
 			 * removing the network here anyway.
 			 */
-			connman_warn("Skipping disconnect of %s",
+			connman_warn("Skipping disconnect of %s, network is connecting.",
 				connman_network_get_identifier(network));
 			continue;
 		}
@@ -957,9 +953,14 @@ int connman_device_set_string(struct connman_device *device,
 	} else if (g_str_equal(key, "Node") == TRUE) {
 		g_free(device->node);
 		device->node = g_strdup(value);
+	} else if (g_str_equal(key, "Path") == TRUE) {
+		g_free(device->path);
+		device->path = g_strdup(value);
+	} else {
+		return -EINVAL;
 	}
 
-	return connman_element_set_string(&device->element, key, value);
+	return 0;
 }
 
 /**
@@ -982,8 +983,10 @@ const char *connman_device_get_string(struct connman_device *device,
 		return device->node;
 	else if (g_str_equal(key, "Interface") == TRUE)
 		return device->interface;
+	else if (g_str_equal(key, "Path") == TRUE)
+		return device->path;
 
-	return connman_element_get_string(&device->element, key);
+	return NULL;
 }
 
 static void set_offlinemode(struct connman_element *element, gpointer user_data)
@@ -1279,7 +1282,6 @@ static int device_load(struct connman_device *device)
 	GError *error = NULL;
 	gchar *identifier;
 	connman_bool_t powered;
-	int val;
 
 	DBG("device %p", device);
 
@@ -1297,12 +1299,6 @@ static int device_load(struct connman_device *device)
 		device->powered_persistent = powered;
 	g_clear_error(&error);
 
-	val = g_key_file_get_integer(keyfile, identifier,
-						"ScanInterval", &error);
-	if (error == NULL)
-		device->scan_interval = val;
-	g_clear_error(&error);
-
 done:
 	g_free(identifier);
 
@@ -1330,9 +1326,6 @@ static int device_save(struct connman_device *device)
 	g_key_file_set_boolean(keyfile, identifier,
 					"Powered", device->powered_persistent);
 
-	g_key_file_set_integer(keyfile, identifier,
-					"ScanInterval", device->scan_interval);
-
 done:
 	g_free(identifier);
 
