@@ -40,6 +40,49 @@
 
 #include "connman.h"
 
+static struct {
+	connman_bool_t bg_scan;
+} connman_settings  = {
+	.bg_scan = TRUE,
+};
+
+static GKeyFile *load_config(const char *file)
+{
+	GError *err = NULL;
+	GKeyFile *keyfile;
+
+	keyfile = g_key_file_new();
+
+	g_key_file_set_list_separator(keyfile, ',');
+
+	if (!g_key_file_load_from_file(keyfile, file, 0, &err)) {
+		connman_error("Parsing %s failed: %s", file, err->message);
+		g_error_free(err);
+		g_key_file_free(keyfile);
+		return NULL;
+	}
+
+	return keyfile;
+}
+
+static void parse_config(GKeyFile *config)
+{
+	GError *error = NULL;
+	gboolean boolean;
+
+	if (config == NULL)
+		return;
+
+	DBG("parsing main.conf");
+
+	boolean = g_key_file_get_boolean(config, "General",
+						"BackgroundScanning", &error);
+	if (error == NULL)
+		connman_settings.bg_scan = boolean;
+
+	g_clear_error(&error);
+}
+
 static GMainLoop *main_loop = NULL;
 
 static void sig_term(int sig)
@@ -117,6 +160,14 @@ const char *connman_option_get_string(const char *key)
 	return NULL;
 }
 
+connman_bool_t connman_setting_get_bool(const char *key)
+{
+	if (g_str_equal(key, "BackgroundScanning") == TRUE)
+		return connman_settings.bg_scan;
+
+	return FALSE;
+}
+
 int main(int argc, char *argv[])
 {
 	GOptionContext *context;
@@ -124,7 +175,7 @@ int main(int argc, char *argv[])
 	DBusConnection *conn;
 	DBusError err;
 	struct sigaction sa;
-	mode_t old_umask;
+	GKeyFile *config;
 
 #ifdef HAVE_CAPNG
 	/* Drop capabilities */
@@ -179,7 +230,7 @@ int main(int argc, char *argv[])
 			perror("Failed to create statistics directory");
 	}
 
-	old_umask = umask(077);
+	umask(0077);
 
 	main_loop = g_main_loop_new(NULL, FALSE);
 
@@ -208,6 +259,10 @@ int main(int argc, char *argv[])
 
 	__connman_dbus_init(conn);
 
+	config = load_config(CONFIGDIR "/main.conf");
+
+	parse_config(config);
+
 	__connman_storage_init();
 	__connman_element_init(option_device, option_nodevice);
 
@@ -219,6 +274,7 @@ int main(int argc, char *argv[])
 	__connman_profile_init();
 	__connman_config_init();
 	__connman_stats_init();
+	__connman_clock_init();
 
 	__connman_resolver_init(option_dnsproxy);
 	__connman_ipconfig_init();
@@ -260,6 +316,7 @@ int main(int argc, char *argv[])
 	__connman_ipconfig_cleanup();
 	__connman_resolver_cleanup();
 
+	__connman_clock_cleanup();
 	__connman_stats_cleanup();
 	__connman_config_cleanup();
 	__connman_profile_cleanup();
@@ -280,5 +337,8 @@ int main(int argc, char *argv[])
 
 	g_main_loop_unref(main_loop);
 
+	if (config)
+		g_key_file_free(config);
+
 	return 0;
 }
